@@ -3,7 +3,7 @@
 // -----------------------------------------------------------------------------
 // This file is part of icinga-web.
 // 
-// Copyright (c) 2009-2012 Icinga Developer Team.
+// Copyright (c) 2009-2013 Icinga Developer Team.
 // All rights reserved.
 // 
 // icinga-web is free software: you can redistribute it and/or modify
@@ -173,7 +173,9 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
             'HOST_OUTPUT'                  =>        'hs.output',
             'HOST_LONG_OUTPUT'             =>        'hs.long_output',
             'HOST_PERFDATA'                =>        'hs.perfdata',
-            'HOST_CURRENT_STATE'           =>        'hs.current_state',
+            // Workaround for missing initial states #3844
+            //'HOST_CURRENT_STATE'           =>        'hs.current_state',
+            'HOST_CURRENT_STATE'           =>        'COALESCE(hs.current_state,0)',
             'HOST_CURRENT_CHECK_ATTEMPT'        =>   'hs.current_check_attempt',
             'HOST_MAX_CHECK_ATTEMPTS'        =>      'hs.max_check_attempts',
             'HOST_LAST_CHECK'              =>        'hs.last_check',
@@ -182,7 +184,9 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
             'HOST_LATENCY'                 =>        'hs.latency',
             'HOST_EXECUTION_TIME'          =>        'hs.execution_time',
             'HOST_NEXT_CHECK'              =>        'hs.next_check',
-            'HOST_HAS_BEEN_CHECKED'        =>        'hs.has_been_checked',
+            // Workaround for missing initial states #3844
+            //'HOST_HAS_BEEN_CHECKED'        =>        'hs.has_been_checked',
+            'HOST_HAS_BEEN_CHECKED'        =>        'COALESCE(hs.has_been_checked,0)',
             'HOST_LAST_HARD_STATE_CHANGE'        =>  'hs.last_hard_state_change',
             'HOST_LAST_NOTIFICATION'        =>       'hs.last_notification',
             'HOST_PROCESS_PERFORMANCE_DATA'        =>       'h.process_performance_data',
@@ -194,8 +198,11 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
             'HOST_STATUS_UPDATE_TIME'          =>    'hs.status_update_time',
             'HOST_ALL'                       =>    'h.*',
             'HOST_STATUS_ALL'                =>      'hs.*',
-            'HOST_STATE'                     =>    'hs.current_state',
+            // Workaround for missing initial states #3844
+            //'HOST_STATE'                     =>    'hs.current_state',
+            'HOST_STATE'                     =>    'COALESCE(hs.current_state,0)',
             'HOST_STATE_COUNT'               =>    'count(hs.current_state)',
+            'HOST_OBJECT_COUNT'               =>    'count(DISTINCT h.host_object_id)',
             'HOST_PARENT_OBJECT_ID'          =>    'ohp.object_id',
             'HOST_PARENT_NAME'               =>    'ohp.name1',
             'HOST_CHILD_OBJECT_ID'           =>    'oh.object_id',
@@ -203,7 +210,9 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
             'HOST_CUSTOMVARIABLE_NAME'         =>        'cvsh.varname',
             'HOST_CUSTOMVARIABLE_VALUE'        =>        'cvsh.varvalue',
             'HOST_CURRENT_PROBLEM_STATE'   =>  '(hs.current_state*(hs.problem_has_been_acknowledged-1)*(hs.scheduled_downtime_depth-1))',
-            'HOST_IS_PENDING'       =>        '(hs.has_been_checked-1)*-1',
+            // Workaround for missing initial states #3844
+            //'HOST_IS_PENDING'       =>        '(hs.has_been_checked-1)*-1',
+            'HOST_IS_PENDING'       =>        '(CASE WHEN hs.has_been_checked IS NULL THEN 1 ELSE (hs.has_been_checked-1)*-1 END)',
             // Service data
 
             'SERVICE_ID'            =>        's.service_id',
@@ -231,7 +240,9 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
             'SERVICE_LONG_OUTPUT'   =>      'ss.long_output',
             'SERVICE_PERFDATA'      =>      'ss.perfdata',
             'SERVICE_PROCESS_PERFORMANCE_DATA'        =>       's.process_performance_data',
-            'SERVICE_CURRENT_STATE' =>      'ss.current_state',
+            // Workaround for missing initial states #3844
+            //'SERVICE_CURRENT_STATE' =>      'ss.current_state',
+            'SERVICE_CURRENT_STATE' =>      'COALESCE(ss.current_state,0)',
             'SERVICE_CURRENT_CHECK_ATTEMPT'=>'ss.current_check_attempt',
             'SERVICE_MAX_CHECK_ATTEMPTS'=>  'ss.max_check_attempts',
             'SERVICE_LAST_CHECK'    =>      'ss.last_check',
@@ -240,7 +251,9 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
             'SERVICE_LATENCY'       =>      'ss.latency',
             'SERVICE_EXECUTION_TIME' =>     'ss.execution_time',
             'SERVICE_NEXT_CHECK'    =>      'ss.next_check',
-            'SERVICE_HAS_BEEN_CHECKED'=>    'ss.has_been_checked',
+            // Workaround for missing initial states #3844
+            //'SERVICE_HAS_BEEN_CHECKED'=>    'ss.has_been_checked',
+            'SERVICE_HAS_BEEN_CHECKED'=>    'COALESCE(ss.has_been_checked,0)',
             'SERVICE_LAST_HARD_STATE'=>     'ss.last_hard_state',
             'SERVICE_LAST_HARD_STATE_CHANGE'=>'ss.last_hard_state_change',
             'SERVICE_LAST_NOTIFICATION'=>   'ss.last_notification',
@@ -255,8 +268,11 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
             'SERVICE_CUSTOMVARIABLE_NAME'=> 'cvss.varname',
             'SERVICE_CUSTOMVARIABLE_VALUE'=>'cvss.varvalue',
             'SERVICE_STATE_COUNT'        => 'count(ss.current_state)',
+            'SERVICE_OBJECT_COUNT'        => 'count(DISTINCT s.service_object_id)',
             'SERVICE_CURRENT_PROBLEM_STATE'   =>  '(ss.current_state*(ss.problem_has_been_acknowledged-1)*(ss.scheduled_downtime_depth-1))',
-            'SERVICE_IS_PENDING'        =>  '(ss.has_been_checked-1)*-1',
+            // Workaround for missing initial states #3844
+            //'SERVICE_IS_PENDING'        =>  '(ss.has_been_checked-1)*-1',
+            'SERVICE_IS_PENDING'       =>     '(CASE WHEN ss.has_been_checked IS NULL THEN 1 ELSE (ss.has_been_checked-1)*-1 END)',
 
             // Config vars
             'CONFIG_VAR_ID'             =>  'cfv.configfilevariable_id',
@@ -444,13 +460,13 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
                         "oc"  => array("src" => "cgm","relation" => "object"),
                         "ocg"  => array("src" => "cg","relation" => "object"),
                         "os"     => array("src" => "s", "relation" => "object"),
-                        "dt"   => array("src" => "h", "relation" => "scheduledDowntimes"),
-                        "cvsh"=> array("src" => "h","relation" => "customvariablestatus"),
-                        "cvsc"=> array("src" => "cgm","relation" => "customvariablestatus"),
-                        "s" => array("src" => "h", "relation" => "services"),
-                        "ss" => array("src" => "s", "relation" => "status"),
+                        "dt"   => array("src" => "h", "relation" => "scheduledDowntimes", "type"=>"left"),
+                        "cvsh"=> array("src" => "h","relation" => "customvariablestatus", "type"=>"left"),
+                        "cvsc"=> array("src" => "cgm","relation" => "customvariablestatus", "type"=>"left"),
+                        "s" => array("src" => "h", "relation" => "services", "type"=>"left"),
+                        "ss" => array("src" => "s", "relation" => "status", "type"=>"left"),
 
-                        "os" => array("src" => "h", "relation" => "object")
+                        "os" => array("src" => "h", "relation" => "object", "type"=>"left")
                 );
                 break;
 
@@ -461,15 +477,15 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
                 $this->aliasDefs = array(
                         "s"  => array("src" => "os", "relation" => "service", "alwaysJoin" => true),
                         "i"  => array("src" => "s", "relation" => "instance"),
-                        "h"  => array("src" => "s","relation" => "host"),
+                        "h"  => array("src" => "s","relation" => "host", "type"=>"left"),
                         "hs" => array("src" => "h","relation" => "status","alwaysJoin" => true, "type"=>"left"),
-                        "oh" => array("src" => "h","relation" => "object"),
+                        "oh" => array("src" => "h","relation" => "object", "type"=>"left"),
                         /*                                       "hcg" => array("src" => "h", "relation" => "contactgroups"),
                                        "hcgm" => array("src" => "h", "relation" => "contacts"),*/
-                        "cg" => array("src" => "s", "relation" => "contactgroups"),
-                        "cgm" => array("src"=> "cg", "relation" => "members"),
-                        "cvsh"=> array("src" => "s","relation" => "customvariablestatus"),
-                        "cvsc"=> array("src" => "cgm","relation" => "customvariablestatus"),
+                        "cg" => array("src" => "s", "relation" => "contactgroups", "type"=>"left"),
+                        "cgm" => array("src"=> "cg", "relation" => "members", "type"=>"left"),
+                        "cvsh"=> array("src" => "s","relation" => "customvariablestatus", "type"=>"left"),
+                        "cvsc"=> array("src" => "cgm","relation" => "customvariablestatus", "type"=>"left"),
                         "ss" => array("src" => "s","relation" => "status","alwaysJoin" => true, "type"=>"left"),
 
                         "sg" => array(
@@ -487,14 +503,14 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
                                 "relation" => "object",
                                 "type"=>"left"
                         ),
-                        "oc"  => array("src" => "cgm","relation" => "object"),
+                        "oc"  => array("src" => "cgm","relation" => "object", "type"=>"left"),
                         "ocg"  => array("src" => "cg","relation" => "object"),
                         "hg"  => array("src" => "h", "relation" => "hostgroups", "type"=>"left"),
                         "hgm" => array("src" => "hg","relation" => "members", "type"=>"left"),
-                        "ohg" => array("src" => "hg","relation" => "object"),
-                        "cvsh" => array("src" => "h","relation"=> "customvariablestatus"),
-                        "cvss"=> array("src" => "s","relation" => "customvariablestatus"),
-                        "cvsc"=> array("src" => "cgm","relation" => "customvariablestatus")
+                        "ohg" => array("src" => "hg","relation" => "object", "type"=>"left"),
+                        "cvsh" => array("src" => "h","relation"=> "customvariablestatus", "type"=>"left"),
+                        "cvss"=> array("src" => "s","relation" => "customvariablestatus", "type"=>"left"),
+                        "cvsc"=> array("src" => "cgm","relation" => "customvariablestatus", "type"=>"left")
                 );
                 break;
 
@@ -505,10 +521,10 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
                 $this->aliasDefs = array(
                         "ohg"   => array("src" => "hg", "relation" => "object"),
                         "hgm"   => array("src" => "hg", "relation" => "members"),
-                        "h"     => array("src" => "hg", "relation" => "members"),
-                        "s"     => array("src" => "h", "relation" => "services"),
-                        "sg"    => array("src" => "s", "relation" => "servicegroups"),
-                        "hs"    => array("src" => "hgm", "relation" => "status"),
+                        "h"     => array("src" => "hg", "relation" => "members", "type"=>"left"),
+                        "s"     => array("src" => "h", "relation" => "services", "type"=>"left"),
+                        "sg"    => array("src" => "s", "relation" => "servicegroups", "type"=>"left"),
+                        "hs"    => array("src" => "hgm", "relation" => "status", "type"=>"left"),
                         "oh"    => array("src" => "hgm", "relation" => "object"),
                         "cg" => array("src" => "hgm", "relation" => "contactgroups"),
                         "cgm" => array("src"=> "cg", "relation" => "members"),
@@ -612,7 +628,7 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
                 $this->retainedAlias = "h";
                 $this->aliasDefs = array(
                         "hs"  => array("src" => "h", "relation" => "status","type"=>"inner","alwaysJoin"=>true),
-                        "s"  => array("src" => "h", "relation" => "services"),
+                        "s"  => array("src" => "h", "relation" => "services", "type"=>"left"),
                         "oh" => array("src" => "h", "relation" => "object", "alwaysJoin" => true, "with"=>"oh.is_active=1"),
                         "i"  => array("src" => "h", "relation" => "instance"),
                         "cg" => array("src" => "h", "relation" => "contactgroups"),
@@ -657,7 +673,7 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
                         "cvsc"=> array("src" => "cgm", "relation" => "customvariablestatus")
                 );
                 break;
-
+            
             case IcingaApiConstants::TARGET_HOST_STATUS_SUMMARY:
                 $this->mainAlias = "h";
                 $this->setTarget("IcingaHosts");
@@ -669,7 +685,7 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
                 $this->retainedAlias = "h";
                 $this->aliasDefs = array(
                         "hs"  => array("src" => "h", "relation" => "status","type"=>"left","alwaysJoin"=>true),
-                        "s"  => array("src" => "h", "relation" => "services"),
+                        "s"  => array("src" => "h", "relation" => "services", "type"=>"left"),
                         "oh" => array("src" => "h", "relation" => "object", "alwaysJoin" => true, "with"=>"oh.is_active=1"),
                         "i"  => array("src" => "h", "relation" => "instance"),
                         "cg" => array("src" => "h", "relation" => "contactgroups"),
@@ -923,16 +939,16 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
                 $this->aliasDefs = array(
                         "i"   => array("src" => "co", "relation" => "instance"),
                         "h" => array("src" => "co", "relation" => "host","type"=>"inner", "alwaysJoin" => true),
-                        "s" => array("src" => "h", "relation" => "service","type"=>"left"),
+                        "s" => array("src" => "h", "relation" => "services","type"=>"left"),
                         "sh" => array("src" => "s", "relation" => "object"),
                         "oh" => array("src" => "h", "relation" => "object","type"=>"left"),
                         "os" => array("src" => "s", "relation" => "object","type"=>"left"),
                         "hs" => array("src" => "h", "relation" => "status"),
                         "ss" => array("src" => "s", "relation" => "status"),
-                        "cg" => array("src" => "h", "relation" => "contactgroups"),
-                        "cgm" => array("src"=> "cg", "relation" => "members"),
-                        "oc"  => array("src" => "cgm","relation" => "object"),
-                        "ocg"  => array("src" => "cg","relation" => "object")
+                        "cg" => array("src" => "h", "relation" => "contactgroups","type"=>"left"),
+                        "cgm" => array("src"=> "cg", "relation" => "members","type"=>"left"),
+                        "oc"  => array("src" => "cgm","relation" => "object","type"=>"left"),
+                        "ocg"  => array("src" => "cg","relation" => "object","type"=>"left")
                 );
                 $this->setTarget("IcingaComments");
                 break;
@@ -982,12 +998,12 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
                                 "relation" => "instance",
                                 "type" => "left"
                         ),
-                        "cg" => array("src" => "s", "relation" => "contactgroups"),
-                        "cgm" => array("src"=> "cg", "relation" => "members"),
-                        "oc"  => array("src" => "cgm","relation" => "object"),
-                        "ocg"  => array("src" => "cg","relation" => "object"),
-                        "cvsh"=> array("src" => "s","relation" => "customvariablestatus"),
-                        "cvsc"=> array("src" => "cgm","relation" => "customvariablestatus"),
+                        "cg" => array("src" => "s", "relation" => "contactgroups", "type"=>"left"),
+                        "cgm" => array("src"=> "cg", "relation" => "members", "type"=>"left"),
+                        "oc"  => array("src" => "cgm","relation" => "object", "type"=>"left"),
+                        "ocg"  => array("src" => "cg","relation" => "object", "type"=>"left"),
+                        "cvsh"=> array("src" => "s","relation" => "customvariablestatus", "type"=>"left"),
+                        "cvsc"=> array("src" => "cgm","relation" => "customvariablestatus", "type"=>"left"),
                         "ss" => array(
                                 "src" => "s",
                                 "relation" => "status",
@@ -1044,9 +1060,9 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
                                 "relation" => "object",
                                 "type" => "left"
                         ),
-                        "cvsh" => array("src" => "h","relation"=> "customvariablestatus"),
-                        "cvss"=> array("src" => "s","relation" => "customvariablestatus"),
-                        "cvsc"=> array("src" => "cgm","relation" => "customvariablestatus")
+                        "cvsh" => array("src" => "h","relation"=> "customvariablestatus", "type"=>"left"),
+                        "cvss"=> array("src" => "s","relation" => "customvariablestatus", "type"=>"left"),
+                        "cvsc"=> array("src" => "cgm","relation" => "customvariablestatus", "type"=>"left")
                 );
                 break;
 
@@ -1164,6 +1180,8 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
             $table = $o->getConnection()->getTable($this->getTarget());
             $keys = $table->getIdentifierColumnNames();
             $o->addSelect(implode($keys));
+        } else {
+            $o->disableAutoIdentifierFields(true);
         }
 
         foreach($this->additionalSelects as $alias => $select) {
@@ -1242,8 +1260,8 @@ class Api_Store_LegacyLayer_TargetModifierModel extends IcingaStoreTargetModifie
             "ohg" => array("src" => "hg","relation" => "object", "type" => "left"),
 
             "hs"  => array("src" => "h", "relation" => "status"),
-            "cvss"=> array("src" => "s","relation" => "customvariablestatus"),
-            "cvsh"=> array("src" => "h", "relation" => "customvariablestatus")
+            "cvss"=> array("src" => "s","relation" => "customvariablestatus","type"=>"left"),
+            "cvsh"=> array("src" => "h", "relation" => "customvariablestatus","type"=>"left")
 
     );
 
