@@ -1,7 +1,7 @@
 <?php
 
 /**
- * Data file for timezone "Etc/GMT+2".
+ * Data file for timezone "Etc/GMT-12".
  * Compiled from olson file "etcetera", version 8.3.
  *
  * @package    agavi
@@ -12,7 +12,7 @@
  *
  * @since      0.11.0
  *
- * @version    $Id: Etc_47_GMT_43_2.php 4640 2011-04-16 11:13:19Z david $
+ * @version    $Id: Etc_47_GMT_45_12.php 4640 2011-04-16 11:13:19Z david $
  */
 
 return array (
@@ -25,13 +25,13 @@ return array (
   'finalRule' => 
   array (
     'type' => 'static',
-    'name' => 'GMT+2',
-    'offset' => -7200,
+    'name' => 'GMT-12',
+    'offset' => 43200,
     'startYear' => -2147483647,
   ),
   'source' => 'etcetera',
   'version' => '8.3',
-  'name' => 'Etc/GMT+2',
+  'name' => 'Etc/GMT-12',
 );
 
 ?>
\ No newline at end of file
